/*
 * mirc.c - communication with mIRC
 * Copyright (c) 2011, D.C. van Moolenbroek, XISE <dc@xise.nl>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 *  * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#define WIN32_LEAN_AND_MEAN
#include <windows.h>

#include "mirc.h"

#define WM_MCOMMAND  (WM_USER + 200)
#define WM_MEVALUATE (WM_USER + 201)

#ifdef UNICODE
#define UNICODE_FLAG 8
#else
#define UNICODE_FLAG 0
#endif

static HANDLE hmap;
static LPTSTR ptr;
static DWORD num;

LPTSTR mirc_open(void)
{
  LARGE_INTEGER li;
  TCHAR name[32];
  DWORD error;

  // get a semi-random number for picking a file mapping name
  QueryPerformanceCounter(&li);
  num = li.LowPart;

  // find a free file mapping to use 
  do {
    // don't use "negative" values, and skip zero
    num = num % 0x7fffffff + 1;

    wsprintf(name, TEXT("mIRC%lu"), num);

    // open the file mapping
    SetLastError(0);

    if ((hmap = CreateFileMapping(INVALID_HANDLE_VALUE, NULL, PAGE_READWRITE,
     0, MIRC_LINE_LENGTH * sizeof(TCHAR), name)) == NULL)
     return NULL;

    error = GetLastError();

    if (error != 0 && error != ERROR_ALREADY_EXISTS)
     return NULL;

    // if this one already exists, try the next number
    if (error)
     CloseHandle(hmap);
  } while (error);

  // map the shared memory onto our address space
  if ((ptr = (TCHAR *) MapViewOfFile(hmap, FILE_MAP_ALL_ACCESS, 0, 0, 0)) ==
   NULL)
    CloseHandle(hmap);

  return ptr;
}

BOOL mirc_send(HWND hwnd)
{
  // tell mIRC to execute the command in the buffer
  return SendMessage(hwnd, WM_MCOMMAND, UNICODE_FLAG, num);
}

BOOL mirc_query(HWND hwnd)
{
  // tell mIRC to evaluate the command in the buffer
  return SendMessage(hwnd, WM_MEVALUATE, UNICODE_FLAG, num);
}

void mirc_close(void)
{
  // unmap and close the filemapping
  UnmapViewOfFile(ptr);

  CloseHandle(hmap);
}
